'use strict';

var webpack = require('webpack'),
    ExtractTextPlugin = require('extract-text-webpack-plugin');

/**
 * Get configuration for Webpack
 *
 * @see http://webpack.github.io/docs/configuration
 *      https://github.com/petehunt/webpack-howto
 *
 * @param {boolean} release True if configuration is intended to be used in
 * a release mode, false otherwise
 * @return {object} Webpack configuration
 */
module.exports = function(release) {
    return {
        entry: {
            app: './src/app.js',
            vendor: [
                'collections/sorted-set',
                'd3',
                'dagre',
                'dagre-d3',
                'event-source-polyfill',
                'fluxxor',
                'lodash',
                'moment',
                'react',
                'react-bootstrap',
                'react-router',
                'react-router-bootstrap',
                'superagent'
            ]
        },

        output: {
            filename: 'app.js',
            path: './build/js/',
            publicPath: '/js/'
        },

        cache: !release,
        debug: !release,
        devtool: 'source-map',

        stats: {
            colors: true,
            reasons: !release
        },

        plugins: release ? [
            new webpack.DefinePlugin({
                'process.env.NODE_ENV': '"production"',
                '__DEV__': false
            }),
            new webpack.optimize.DedupePlugin(),
            new webpack.optimize.UglifyJsPlugin(),
            new webpack.optimize.OccurenceOrderPlugin(),
            new webpack.optimize.AggressiveMergingPlugin(),
            new webpack.optimize.CommonsChunkPlugin('vendor', 'vendor.bundle.js'),
            new ExtractTextPlugin('styles.css')
        ] : [
            new webpack.DefinePlugin({'__DEV__': true}),
            new webpack.optimize.CommonsChunkPlugin('vendor', 'vendor.bundle.js'),
            new ExtractTextPlugin('styles.css')
        ],

        resolve: {
            extensions: ['', '.webpack.js', '.web.js', '.js']
        },

        module: {
            loaders: [
                {
                    test: /\.less$/,
                    loader: ExtractTextPlugin.extract(
                        'css-loader?sourceMap!autoprefixer-loader?{browsers:[' +
                            '"Android 2.3", "Android >= 4", "Chrome >= 20", "Firefox >= 24", ' +
                            '"Explorer >= 8", "iOS >= 6", "Opera >= 12", "Safari >= 6"]}!' +
                            'less-loader?sourceMap'
                    )
                },
                {
                    test: /\.gif/,
                    loader: 'url-loader?limit=10000&mimetype=image/gif'
                },
                {
                    test: /\.jpg/,
                    loader: 'url-loader?limit=10000&mimetype=image/jpg'
                },
                {
                    test: /\.png/,
                    loader: 'url-loader?limit=10000&mimetype=image/png'
                },
                {
                    test: /\.woff(\?v=\d+\.\d+\.\d+)?$/,
                    loader: 'file-loader?minetype=application/font-woff'
                },
                {
                    test: /\.woff2(\?v=\d+\.\d+\.\d+)?$/,
                    loader: 'file-loader?minetype=application/font-woff2'
                },
                {
                    test: /\.ttf(\?v=\d+\.\d+\.\d+)?$/,
                    loader: 'file-loader'
                },
                {
                    test: /\.eot(\?v=\d+\.\d+\.\d+)?$/,
                    loader: 'file-loader'
                },
                {
                    test: /\.svg(\?v=\d+\.\d+\.\d+)?$/,
                    loader: 'file-loader'
                },
                {
                    test: /\.js?$/,
                    exclude: /node_modules/,
                    loader: 'babel-loader?optional=runtime'
                }
            ]
        }
    };
};
