'use strict';

var g = require('gulp'),
    p = require('gulp-load-plugins')(),
    argv = require('minimist')(process.argv.slice(2)),
    webpack = require('webpack'),
    runSequence = require('run-sequence'),
    del = require('del');

var RELEASE = !!argv.release,
    watch = false,
    paths = {
        js: 'src/**/*.js',
        images: 'src/images/*',
        pages: 'src/pages/*.html'
    };

var src = {};

g.task('clean', function(cb) {
    return del(['build'], cb);
});

g.task('vendor', function() {
    return g
        .src('src/vendor/*.js')
        .pipe(g.dest('build/js'))
        .pipe(p.size({title: 'vendor'}));
});

g.task('bundle', function(cb) {
    var started = false,
        config = require('./config/webpack.js')(RELEASE),
        bundler = webpack(config);

    function bundle(err, stats) {
        if (err) {
            throw new p.util.PluginError('webpack', err);
        }

        var errors = stats.toJson({errorDetails: true}).errors;
        if (errors.length > 0) {
            p.util.log('[webpack]', errors[0]);
        }
        else {
            p.util.log('bundle built');
        }
        
        if (argv.verbose) {
            p.util.log('[webpack]', stats.toString({colors: true}));
        }

        if (!started) {
            started = true;
            return cb();
        }

        return false;
    }

    if (watch) {
        bundler.watch(200, bundle);
    } else {
        bundler.run(bundle);
    }
});

g.task('images', function() {
    return g
        .src(paths.images)
        .pipe(g.dest('build/images'))
        .pipe(p.size({title: 'images'}));
});

g.task('pages', function() {
    return g
        .src(paths.pages)
        .pipe(g.dest('build'))
        .pipe(p.size({title: 'pages'}));
});

g.task('watch', function(cb) {
    watch = true;

    runSequence('bundle');

    cb();

    console.log('Watching files for changes...');
});

g.task('build', function(cb) {
    runSequence('clean', ['bundle', 'vendor', 'images', 'pages'], cb);
});

g.task('default', ['build']);
