var React = require('react');

import {map, isString} from 'lodash';

var ResourceLink = require('../generic/resource_link'),
    RoleLink = require('../generic/role_link');

var FieldsMixin = require('./fields_mixin'),
    Timestamp = require('./timestamp'),
    humanizeEvent = require('./humanize_event');

export default React.createClass({
    displayName: 'AuditEntry',

    mixins: [FieldsMixin],

    propTypes: {
        compact: React.PropTypes.bool
    },

    getDefaultProps() {
        return {
            compact: false
        };
    },

    render() {
        var items = map(this.fields(), (field) => {
            var item = this.transformField(field, this.props[field]);

            return (
              <td key={field}>
                {item}
              </td>
            );
        });

        return (
          <tr className={this.props.action}>
            {items}
          </tr>
        );
    },

    transformField(key, value) {
        var ret = [''];

        switch (key) {
        case 'entities':
            ret = [
                isString(this.props.resource) ?
                    <ResourceLink id={this.props.resource.id} /> : null,
                isString(this.props.role) ?
                    <RoleLink id={this.props.role} /> : null
            ];

            break;

        case 'user':
            if (isString(value)) {
                ret = [<RoleLink id={value} />];
            }

            break;

        case 'acting_as':
            if ((isString(value)) && (value !== this.props.user)) {
                ret = [<RoleLink id={value} />];
            }

            break;

        case 'auditview_user': //virtual field
            var actingUser = this.props.user;
            var actingRole = this.props.acting_as;

            ret = [<RoleLink id={actingUser} />];

            if ((isString(actingRole)) && (actingRole !== actingUser)) {
                ret.push([
                    ' as ',
                    <RoleLink id={actingRole} />
                ]);
            }

            break;

        case 'timestamp':
            if (isString(value)) {
                ret = [<Timestamp time={value} />];
            }

            break;

        case 'auditview_action':
            ret = humanizeEvent(this.props);
            break;

        default:
            ret = [value];
        }

        return ret;
    }
});
