import React, {Component, PropTypes} from 'react';
import {readCookie} from '../../utils';

import './login.less';

export class Login extends Component {
    constructor() {
        super();
        this.handleChange = this.handleChange.bind(this);
        this.handleSubmit = this.handleSubmit.bind(this);
        this.onLoginSuccess = this.onLoginSuccess.bind(this);
        this.onLoginError = this.onLoginError.bind(this);
        this.state = {
            error: false,
            isSubmiting: false,
            applianceUrl: readCookie('conjur_appliance_url') || document.location.origin,
            username: readCookie('conjur_username') || '',
            password: readCookie('conjur_password') || ''
        };
    }

    componentDidMount() {
        const rmanager = this.context.flux.actions.getRManager();

        if (rmanager.isLogged) {
            this.redirect();
        }
    }

    onLoginSuccess() {
        const rmanager = this.context.flux.actions.getRManager();

        if (rmanager.isLogged) {
            this.redirect();
        }

        this.setState({error: false, isSubmiting: false});
    }

    onLoginError() {
        this.setState({error: true, isSubmiting: false});
    }

    handleChange(e) {
        e.preventDefault();

        this.setState({
            username: this.refs.username.getDOMNode().value,
            password: this.refs.password.getDOMNode().value
        });
    }

    handleSubmit(e) {
        e.preventDefault();

        if (!this.state.isSubmiting) {
            const rmanager = this.context.flux.actions.getRManager();

            rmanager
                .login(
                    this.state.applianceUrl,
                    this.state.username,
                    this.state.password
                )
                .then(
                    (isSuccess) => {
                        if (isSuccess) {
                            this.onLoginSuccess();
                        } else {
                            this.onLoginError();
                        }
                    }
                );
            this.setState({isSubmiting: true});
        }
    }

    redirect() {
        let {router} = this.context,
            returnTo = router.getCurrentQuery().returnTo || '/ui';

        window.setTimeout(() => {
            router.replaceWith(returnTo);
        }, 1);
    }

    render() {
        const error = this.state.error === true && (<p>Bad login information</p>);

        return (
          <div className="b-login">
            <div className="b-login--row">
              <div className="container-fluid b-login--grid">
                <div className="row">
                  <div className="col-sm-12">
                    <div className="b-login--wrapper">
                      <div className="b-login--inner-wrapper">
                        <div className="b-login__box">
                          <div className="b-login__title-box text-center">
                            <h3 className="b-login__title">
                              Sign in to Conjur
                            </h3>
                          </div>
                          <div>
                            <div className="b-login__fields">
                              <div className="login-info">
                              Login to <strong>{this.state.applianceUrl}</strong> server
                              </div>
                              <form role="form" autoComplete="off">
                                <div className="form-group">
                                  <div className="input-group input-group-lg">
                                    <div className="input-group-addon">
                                      <span className="glyphicon glyphicon-user" aria-hidden="true"></span>
                                    </div>
                                    <input className="form-control"
                                           type="text"
                                           placeholder="user name"
                                           ref="username"
                                           onChange={this.handleChange}
                                           value={this.state.username}
                                           disabled={this.state.isSubmiting} />
                                  </div>
                                </div>
                                <div className="form-group">
                                  <div className="input-group input-group-lg">
                                    <div className="input-group-addon">
                                      <span className="glyphicon glyphicon-pencil" aria-hidden="true"></span>
                                    </div>
                                    <input className="form-control"
                                           type="password"
                                           placeholder="password"
                                           ref="password"
                                           onChange={this.handleChange}
                                           value={this.state.password}
                                           disabled={this.state.isSubmiting} />
                                  </div>
                                </div>
                                <div className="form-group">
                                  <div className="container-fluid">
                                    <div className="row">
                                      <div className="col-xs-8">
                                        {error}
                                      </div>
                                      <div className="col-xs-4 text-right"
                                           style={{paddingRight: '0'}}>
                                        <button role="button"
                                                className="btn b-login__btn"
                                                type="submit"
                                                onClick={this.handleSubmit}
                                                disabled={this.state.isSubmiting}>
                                          Login
                                        </button>
                                      </div>
                                    </div>
                                  </div>
                                </div>
                              </form>
                            </div>
                          </div>
                        </div>
                      </div>
                    </div>
                  </div>
                </div>
              </div>
            </div>
          </div>
        );
    }
}

Login.displayName = 'AuthLogin';

Login.contextTypes = {
    flux: PropTypes.object,
    router: PropTypes.func
};
