var React = require('react'),
    Fluxxor = require('fluxxor');

var FluxMixin = Fluxxor.FluxMixin(React),
    StoreWatchMixin = Fluxxor.StoreWatchMixin;

var AuditTable = require('../audit/table'),
    Annotations = require('../generic/annotations'),
    Breadcrumbs = require('../generic/breadcrumbs'),
    Refresh = require('../refresh/refresh');

export default React.createClass({
    displayName: 'CustomView',

    mixins: [FluxMixin, StoreWatchMixin('audit', 'resources')],

    askForNewData(state) {
        var actions = this.getFlux().actions;

        actions.audit.loadForResource(state.kind, state.id);
        actions.resources.loadOne(state.kind, state.id);
    },

    componentDidMount() {
        this.askForNewData(this.state);
    },

    componentWillUpdate(nextProps, nextState) {
        if ((this.state.kind !== nextState.kind) ||
            (this.state.id !== nextState.id)) {
            this.askForNewData(nextState);
        }

        return true;
    },

    getStateFromFlux() {
        var flux = this.getFlux(),
            kind = flux.store('route').getParam('kind'),
            id = flux.store('route').getParam('id'),
            audit = flux.store('audit').getEventFor(kind, id),
            resource = flux.store('resources').getResource(kind, id);

        return {
            kind: kind,
            id: id,
            loading: {
                audit: audit.loading,
                resource: resource.loading
            },
            error: {
                resource: resource.error
            },
            audit: audit.data,
            resource: resource.data
        };
    },

    render() {
        var elems = [
            {url: '/ui/custom-types', text: 'Custom Types'},
            {url: '', text: this.state.resource.id},
            {url: '', text: '(owned by ' + this.state.resource.owner + ')'}
        ];

        return (
          <div className="b-variable">
            <Breadcrumbs elems={elems} />
            <hr />
            <h2>Custom Type {this.state.resource.kind}:{this.state.resource.identifier}</h2>
            <hr/>
            <h2>Annotations<Refresh show={this.state.loading.resource} /></h2>
            <Annotations annotations={this.state.resource.annotations} />
            <hr />
            <AuditTable events={this.state.audit}
                        caption="Recent Activity"
                        isLoading={this.state.loading.audit} />
          </div>
        );
    }
});
