var React = require('react');

var Chart = require('../chart/chart'),
    ChartHelperMixin = require('../chart/chart_helper_mixin'),
    Refresh = require('../refresh/refresh');

export default React.createClass({
    displayName: 'DashboardActivity',

    mixins: [ChartHelperMixin],

    getDefaultProps() {
        return {
            options: {
                legend: {
                    slogins: 'Logins',
                    ssudo: 'Sudo calls',
                    sreads: 'Secret reads',
                    supdates: 'Secret updates',
                    fsudo: 'Sudo failures',
                    freads: 'Secret read failures',
                    fupdates: 'Secret update failures',
                    fother: 'Other failures'
                },
                axis: {
                    y: {
                        label: 'Value'
                    }
                }
            }
        };
    },

    getDefaultItem() {
        return {
            slogins: 0,
            ssudo: 0,
            fsudo: 0,
            sreads: 0,
            freads: 0,
            supdates: 0,
            fupdates: 0,
            fother: 0
        };
    },

    _getEventTypeSsh(e) {
        if (e.allowed === true) {
            if (e.action === 'sudo') {
                return 'ssudo';
            }

            if (e.action === 'login') {
                return 'slogins';
            }
        }

        if (e.allowed === false) {
            if (e.action === 'sudo') {
                return 'fsudo';
            }
        }

        return null;
    },

    _getEventTypeVariabe(e) {
        if (e.hasOwnProperty('error') || e.allowed === false) {
            if (e.privilege === 'execute') {
                return 'freads';
            }

            if (e.privilege === 'update') {
                return 'fupdates';
            }
        }

        if (!e.hasOwnProperty('error') && e.allowed !== false) {
            if (e.privilege === 'execute') {
                return 'sreads';
            }

            if (e.privilege === 'update') {
                return 'supdates';
            }
        }

        return null;
    },

    getEventType(e) {
        let ret = null;

        if (e.hasOwnProperty('facility') && e.facility === 'ssh') {
            ret = this._getEventTypeSsh(e);
        }

        if ((e.action === 'check') &&
            e.hasOwnProperty('resource') &&
            e.resource.split(':')[1] === 'variable') {
            ret = this._getEventTypeVariabe(e);
        }

        if (e.hasOwnProperty('error') || e.allowed === false) {
            ret = 'fother';
        }

        return ret;
    },

    getInitialState() {
        return {showActivity: false};
    },

   toggleActivity(e) {
        e.preventDefault();

        this.setState({showActivity: !this.state.showActivity});
    },

    render() {
        var data = this.getData(this.props.audit);
        var activityBody = [];

        var msg = this.state.showActivity ? 'Hide' : 'Show graph';

        if (this.state.showActivity) {
            activityBody.push(
                <div className="b-dashboard-activity__graph">
                  <Chart options={this.props.options}
                         data={data} />
                </div>
            );
        }

        return (
          <div className="b-dashboard-activity">
            <h2>Activity<Refresh show={this.props.isLoading} /></h2>
            <a href="#" onClick={this.toggleActivity}>{msg}</a>
            {activityBody}
          </div>
        );
    }
});
