var React = require('react');

var TabPane = require('react-bootstrap/lib/TabPane');

var OwnedResources = require('../owned_resources/owned_resources'),
    RoleLink = require('./role_link'),
    Annotations = require('./annotations'),
    Permissions = require('../permissions/permissions');

import {getTabname} from '../../utils';
import {includes} from 'lodash';

// we can't create custom TabPanes as components,
// because they won't be recognized by TabbedArea as tabs
// that's why we just generalize their constructors
export default {
    containsOwnedItems() {
        return (
            this.state.owned &&
                this.state.owned.length &&
                this.state.owned.length > 0
        );
    },

    ownedIds() {
        if (this.containsOwnedItems()) {
            return this.state.owned.map(function(ownedItem) {
                return ownedItem.id;
            });
        }

        return [];
    },

    ownedTab() {
        if (this.containsOwnedItems()) {
            var result = (
              <TabPane eventKey="owned"
                       tab={getTabname('Owned resources', this.state.owned)}>
                <OwnedResources owned={this.state.owned} tabview={true} />
              </TabPane>
            );

            return result;
        }

        return null;
    },

    containsNontrivialRoles() {
        return (
            this.state.roles &&
                this.state.roles.length &&
                this.state.roles.length > 1
        );
    },

    externalRoles(ownId) {
        if (!this.containsNontrivialRoles()) {
            return [];
        }

        var ownedIds = this.ownedIds();

        return this.state.roles.filter(function(roleid) {
            // do not show themself
            return roleid !== ownId;
        }).filter(function(roleid) {
            // do not show internal roles
            return roleid.split(':')[1] !== '@';
        }).filter(function(roleid) {
            // do not show owned groups
            return !includes(ownedIds, roleid);
        });
    },

    membershipsTab(ownId) {
        if (!this.containsNontrivialRoles()) {
            return null;
        }

        var membershipLinks = this.externalRoles(ownId)
                .sort(function(a, b) {
                    return a.toLowerCase().localeCompare(b.toLowerCase());
                })
                .map(function(roleid) {
                    return (
                      <li className="list-group-item list-group-item-noborder">
                        <RoleLink id={roleid} />
                      </li>
                    );
                });

        if (membershipLinks.length === 0) {
            return null;
        }

        var result = (
          <TabPane eventKey="memberships"
                   tab={getTabname('Explicit memberships', membershipLinks)}>
            <ul className="list-group">
              {membershipLinks}
            </ul>
          </TabPane>
        );

        return result;
    },

    containsAnnotations() {
        return (
            this.state.annotations &&
                this.state.annotations.length &&
                this.state.annotations.length > 0
        );
    },

    annotationsTab() {
        if (!this.containsAnnotations()) {
            return null;
        }

        var result = (
          <TabPane eventKey="annotations"
                   tab={getTabname('Annotations', this.state.annotations)}>
            <Annotations annotations={this.state.annotations} />
          </TabPane>
        );

        return result;
    },

    permissionsTab(roleid) {
        var result = (
          <TabPane eventKey="permissions" tab="Permissions held">
            <Permissions owned={this.state.owned}
                         roles={this.state.roles}
                         role={roleid}
                         resources={this.state.resources}
                         tabview={true} />
          </TabPane>
        );

        return result;
    }
};
