/* global jest, describe, it, expect */
"use strict";


jest.dontMock('lodash');
jest.dontMock('collections/set');
jest.dontMock('../helpers');

var {find} = require('lodash');
var Set = require('collections/set');
var {collapse, edgesFromVertices, __test__:{SingleVertex}} = require('../helpers');

/*
    The graph has the following structure:

    user:admin -> group:a group:b group:c group:d user:a user:b layer:e
    group:a, group:b, group:c -> user:c, user:d
    group:d -> user:e
    user:c, user:d -> user:e

    When collapsed it should look like

    user:admin -> [3 groups (a,b,c)], group:d, [2 users (a,b)], layer:e
    [3 groups(a,b,c)] -> [2 users(c,d)]
    [2 users(c,d)] -> user:e
    group:d -> user:e

 */


// helpers
function shouldContain(set_, predicate){
    expect(find(set_.toArray(), predicate)).toBe(true);
}

function shouldHaveChild(vertex, predicate){
    shouldContain(vertex.out, predicate);
}

function shouldHaveParent(vertex, predicate){
    shouldContain(vertex.in, predicate);
}

function shouldContainMultis(set_, idLists){
    idLists.forEach(
        (idList) => {
            expect(find(set_.toArray(),
                (v) => {
                    if(v.isSingle()) return false;
                    return v.members.equals(new Set(idList));
                }
            )).toBeTruthy()
        }
    );
}

function shouldContainSingles(set_, ids){
    ids.forEach(
        (id) => {
            expect(set_.has(new SingleVertex(id))).toBeTruthy();
        }
    )
}

class GraphBuilder {
    constructor(){
        this._vs = {}; // maps id to vertex
    }

    connect(parentIds, childIds){
        parentIds = typeof parentIds === 'string' ? [parentIds] : parentIds;
        childIds = typeof childIds === 'string' ? [childIds] : childIds;
        parentIds.forEach(
            (pId) => {
                var p = this._vertex(pId);
                childIds.forEach(
                    (cId) => {
                        p.addChild(this._vertex(cId));
                    }
                );
            }
        );
        return this;
    }

    vertices(){
        return Object.values(this._vs);
    }

    edges(){
        return edgesFromVertices(this.vertices());
    }

    _vertex(id){
        return this._vs[id] || (this._vs[id] = new SingleVertex(id));
    }
}


describe("collapsing a simple graph", ()=>{
    var initialGraph = new GraphBuilder()
        .connect('user:admin', ['group:a', 'group:b', 'group:c', 'group:d',
            'user:a',  'user:b',
            'layer:e'])
        .connect(['group:a', 'group:b', 'group:c'], ['user:c', 'user:d'])
        .connect('group:d', 'user:e')
        .connect(['user:c', 'user:d'], 'user:e')
        .vertices();

    var collapsed = collapse(initialGraph, 1);
})


//console.dir(collapsed.map((v) => v.toString()));
//collapsed.forEach(
//    (v) => {
//        console.log('v = ' + v.toString());
//        console.log('   in: ');
//        v.in.forEach(
//            (vIn) => {
//                console.log('       ' + vIn.toString())
//            }
//        );
//        console.log('   out: ');
//        v.out.forEach(
//            (vOut) => {
//                console.log('       ' + vOut.toString())
//            }
//        );
//        console.log('-----------------');
//        console.log('');
//    }
//);