/* global jest, describe, it, expect */
"use strict";


jest.dontMock('lodash');
jest.dontMock('collections/set');
jest.dontMock('../helpers');

var {edgesFromVertices, __test__:{SingleVertex}} = require('../helpers');
var Set = require('collections/set');

describe('edgesFromVertices', ()=>{

    var a = new SingleVertex('x:a'),
        b = new SingleVertex('x:b'),
        c = new SingleVertex('x:c'),
        d = new SingleVertex('x:d');
    a.addChild(b);
    a.addChild(c);
    b.addChild(d);
    c.addChild(d);
    var vertices = [a,b,c,d];

    var edges = edgesFromVertices(vertices),
        edgeSet = new Set(
            edges,
            (a,b) => a.parent === b.parent && a.child === b.child,
            (a) => `${a.parent}->${a.child}`
        ) ;


    function expectEdge(parent, child){
        expect(edgeSet.has({parent, child})).toBeTruthy();
    }

    it('has the expected edges', ()=>{
        expectEdge('x:a', 'x:b');
        expectEdge('x:a', 'x:c');
        expectEdge('x:b', 'x:d');
        expectEdge('x:c', 'x:d');
    });

    it('has the expected number of edges', ()=>{
        expect(edges.length).toBe(4);
    })


});