/* global jest, describe, it, expect */
"use strict";


jest.dontMock('lodash');
jest.dontMock('collections/set');
jest.dontMock('../helpers');

var {verticesFromEdges} = require('../helpers');
var {reduce} = require('lodash');


describe('verticesFromEdges', ()=> {
    /*
     x:c
     |
     v
     x:a
     /     |
     x:b    x:d
     \     /
     x:e
     */
    var edges = [
        ['x:a', 'x:b'],
        ['x:c', 'x:a'],
        ['x:a', 'x:d'],
        ['x:d', 'x:e'],
        ['x:b', 'x:e']
    ];

    var vertices = verticesFromEdges(edges);


    var idToVertex = reduce(vertices,
        (o, v) => {
            o[v.id] = v;
            return o;
        },
        {}
    );

    function expectStructure(vertexId, expectedInIds, expectedOutIds){
        var vertex = idToVertex[vertexId];
        expect(vertex.in.length).toBe(expectedInIds.length);
        expectedInIds.forEach(
            (vId) => {
                let v = idToVertex[vId];
                expect(vertex.in.has(v)).toBeTruthy();
            }
        );
        expectedOutIds.forEach(
            (vId) => {
                let v = idToVertex[vId];
                expect(vertex.out.has(v)).toBeTruthy();
            }
        );
    }

    it('makes 5 vertices', ()=> {
        expect(vertices.length).toBe(5);
    });

    it('creates the correct structure', () => {
        expectStructure('x:c', [], ['x:a']);
        expectStructure('x:a', ['x:c'], ['x:b', 'x:d']);
        expectStructure('x:b', ['x:a'], ['x:e']);
        expectStructure('x:d', ['x:a'], ['x:e']);
        expectStructure('x:e', ['x:b', 'x:d'], []);
    });

});