var React = require('react'),
    Fluxxor = require('fluxxor');

var FluxMixin = Fluxxor.FluxMixin(React),
    StoreWatchMixin = Fluxxor.StoreWatchMixin;

var TabbedArea = require('react-bootstrap/lib/TabbedArea'),
    TabPane = require('react-bootstrap/lib/TabPane');

var TabMixin = require('../generic/tab_mixin'),
    Breadcrumbs = require('../generic/breadcrumbs'),
    RoleLink = require('../generic/role_link'),
    RoleGraph = require('../graph/graph');

import {getTabname} from '../../utils';

var AuditTable = require('../audit/table');

import {compact, map} from 'lodash';

export default React.createClass({
    displayName: 'GroupView',

    mixins: [FluxMixin, StoreWatchMixin('audit', 'group', 'resources', 'route'), TabMixin],

    askForNewData(state) {
        var actions = this.getFlux().actions;

        actions.audit.loadForRole('group', state.id);
        actions.audit.loadForResource('group', state.id);
        actions.group.load(state.id);
        actions.resources.loadOne('group', state.id);
    },

    componentDidMount() {
        this.askForNewData(this.state);
    },

    componentWillUpdate(nextProps, nextState) {
        if (this.state.id !== nextState.id) {
            this.askForNewData(nextState);
        }

        return true;
    },

    getStateFromFlux() {
        var flux = this.getFlux(),
            id = flux.store('route').getParam('id'),
            audit = flux.store('audit').getEventFor('group', id),
            data = flux.store('group').getData(),
            resource = flux.store('resources').getResource('group', id);

        var ret = {
            id: id,
            loading: data.loading,
            group: data.group,
            owned: data.owned,
            roles: data.roles,
            owner: resource.data.owner,
            annotations: resource.data.annotations,
            resource: resource.data,
            members: data.members,
            resources: data.resources,
            audit: audit.data
        };

        ret.loading.audit = audit.loading;
        ret.loading.owner = resource.loading;
        ret.loading.annotations = resource.loading;

        return ret;
    },

    render() {
        var members = map(this.state.members, function(member) {
            return (
              <li className="list-group-item list-group-item-noborder">
                <RoleLink id={member.member} />
              </li>
            );
        });

        // TODO: refactor (common code should go to mixins)
        // TODO: memberships
        // TODO: controls for members management
        var overviewTab = (
          <TabPane eventKey="overview" tab="Overview">
            <dl className="dl-horizontal">
              <dt>Created by</dt>
              <dd><RoleLink id={this.state.group.userid} /></dd>

              <dt>Owner</dt>
              <dd><RoleLink id={this.state.owner} /></dd>
            </dl>
          </TabPane>
        );

        var membersTab = (
          <TabPane eventKey="members" tab={getTabname('Members', members)}>
            <ul className="list-group">
              {members}
            </ul>
          </TabPane>
        );

        var ownedTab = this.ownedTab(),
            membershipsTab = this.membershipsTab(this.state.group.roleid),
            permissionsTab = this.permissionsTab(this.state.group.roleid),
            annotationsTab = this.annotationsTab();

        var auditTab = (
          <TabPane eventKey="audit" tab="Recent Activity">
            <div className="audit auditGroup">
              <AuditTable events={this.state.audit}
                          caption=""
                          isLoading={this.state.loading.audit} />
            </div>
          </TabPane>
        );

        var tabs = compact([
            overviewTab,
            membersTab,
            ownedTab,
            membershipsTab,
            permissionsTab,
            annotationsTab,
            auditTab
        ]);

        var elems = [
            {url: '/ui/groups', text: 'Groups'},
            {url: '', text: this.state.group.id},
            {url: '', text: '(owned by ' + this.state.group.ownerid + ')'}
        ];

        return (
          <div className="b-group">
            <Breadcrumbs elems={elems} />
            <hr />
            <h2>Group {this.state.group.id}</h2>
            <TabbedArea defaultActiveKey="overview">
              {tabs}
            </TabbedArea>
            <hr />
            <RoleGraph height="400"
                       kind="group"
                       id={this.state.id} />
          </div>
        );
    }
});
