var React = require('react'),
    Fluxxor = require('fluxxor');

import {compact} from 'lodash';

var FluxMixin = Fluxxor.FluxMixin(React),
    StoreWatchMixin = Fluxxor.StoreWatchMixin;

var TabbedArea = require('react-bootstrap/lib/TabbedArea'),
    TabPane = require('react-bootstrap/lib/TabPane');

var AuditTable = require('../audit/table'),
    Breadcrumbs = require('../generic/breadcrumbs'),
    RoleLink = require('../generic/role_link'),
    TabMixin = require('../generic/tab_mixin'),
    HostLink = require('../host/host_link'),
    RoleGraph = require('../graph/graph');

import {getTabname} from '../../utils';

var abstractRole = function(expected) {
    return function(role) {
        var tokens = role.split(':'),
            kind = tokens[1],
            abstractKinds = ['@', 'layer'],
            isAbstract = abstractKinds.indexOf(kind) !== -1;

        return isAbstract === expected;
    };
};

export default React.createClass({
    displayName: 'LayerView',

    mixins: [FluxMixin, StoreWatchMixin('audit', 'layer', 'resources', 'route'), TabMixin],

    askForNewData(state) {
        var actions = this.getFlux().actions;

        actions.audit.loadForRole('layer', state.id);
        actions.audit.loadForResource('layer', state.id);
        actions.layer.load(state.id);
        actions.resources.loadOne('layer', state.id);
    },

    componentDidMount() {
        this.askForNewData(this.state);
    },

    componentWillUpdate(nextProps, nextState) {
        if (this.state.id !== nextState.id) {
            this.askForNewData(nextState);
        }

        return true;
    },

    getStateFromFlux() {
        var flux = this.getFlux(),
            id = flux.store('route').getParam('id'),
            audit = flux.store('audit').getEventFor('layer', id),
            data = flux.store('layer').getData(),
            resource = flux.store('resources').getResource('layer', id);

        var ret = {
            id: id,
            loading: data.loading,
            layer: data.layer,
            owned: data.owned,
            roles: data.roles,
            owner: resource.data.owner,
            annotations: resource.data.annotations,
            resource: resource.data,
            users: data.users,
            admins: data.admins,
            resources: data.resources,
            audit: audit.data
        };

        ret.loading.audit = audit.loading;
        ret.loading.owner = resource.loading;
        ret.loading.annotations = resource.loading;

        return ret;
    },

    render() {
        var hosts = this.state.layer.hosts.map(function(host) {
            return (
              <li className="list-group-item list-group-item-noborder">
                <HostLink data={host} />
              </li>
            );
        });

        var admins = this.state.admins.filter(abstractRole(false)).map(function(role) {
            return (
              <li className="list-group-item list-group-item-noborder">
                <RoleLink id={role}/>
              </li>
            );
        });

        var users = this.state.users.filter(abstractRole(false)).map(function(role) {
            return (
              <li className="list-group-item list-group-item-noborder">
                <RoleLink id={role}/>
              </li>
            );
        });

        // TODO: controls to add/remove admins/users/hosts
        var overviewTab = (
          <TabPane eventKey="overview" tab="Overview">
            <dl className="dl-horizontal">
              <dt>Owner</dt>
              <dd><RoleLink id={this.state.owner} /></dd>

              <dt>Created by</dt>
              <dd><RoleLink id={this.state.layer.userid} /></dd>
            </dl>
          </TabPane>
        );

        var adminsTab = (
          <TabPane eventKey="admins" tab={getTabname('Admins', admins)}>
            <ul className="list-group">
              {admins}
            </ul>
          </TabPane>
        );

        var usersTab = (
          <TabPane eventKey="users" tab={getTabname('Users', users)}>
            <ul className="list-group">
              {users}
            </ul>
          </TabPane>
        );

        var hostsTab = (
          <TabPane eventKey="hosts" tab={getTabname('Hosts', hosts)}>
            <ul className="list-group">
              {hosts}
            </ul>
          </TabPane>
        );

        var auditTab = (
          <TabPane eventKey="audit" tab="Recent Activity">
            <div className="audit auditGroup">
              <AuditTable events={this.state.audit}
                          caption=""
                          isLoading={this.state.loading.audit} />
            </div>
          </TabPane>
        );

        var permissionsTab = this.permissionsTab(this.state.layer.roleid),
            membershipsTab = this.membershipsTab(this.state.layer.roleid),
            annotationsTab = this.annotationsTab(),
            ownedTab = this.ownedTab();

        var tabs = compact([
            overviewTab,
            adminsTab,
            usersTab,
            hostsTab,
            ownedTab,
            membershipsTab,
            permissionsTab,
            annotationsTab,
            auditTab
        ]);

        var elems = [
            {url: '/ui/layers', text: 'Layers'},
            {url: '', text: this.state.layer.id},
            {url: '', text: '(owned by ' + this.state.layer.ownerid + ')'}
        ];

        return (
          <div className="b-layer">
            <Breadcrumbs elems={elems} />
            <hr />
            <h2>Layer {this.state.layer.id}</h2>
            <TabbedArea defaultActiveKey="overview">
              {tabs}
            </TabbedArea>
            <hr />
            <RoleGraph height="400"
                       kind="layer"
                       id={this.state.id} />
          </div>
        );
    }
});
