var React = require('react'),
    Fluxxor = require('fluxxor');

import {compact} from 'lodash';

var FluxMixin = Fluxxor.FluxMixin(React),
    StoreWatchMixin = Fluxxor.StoreWatchMixin;

var TabbedArea = require('react-bootstrap/lib/TabbedArea'),
    TabPane = require('react-bootstrap/lib/TabPane');

var TabMixin = require('../generic/tab_mixin'),
    RoleLink = require('../generic/role_link'),
    AuditTable = require('../audit/table');

export default React.createClass({
    displayName: 'PolicyView',

    mixins: [FluxMixin, StoreWatchMixin('policy', 'route'), TabMixin],

    askForNewData(state) {
        var actions = this.getFlux().actions;

        actions.policy.load(state.id);
        actions.resources.loadOne('policy', state.id);
    },

    componentDidMount() {
        this.askForNewData(this.state);
    },

    componentWillUpdate(nextProps, nextState) {
        if (this.state.id !== nextState.id) {
            this.askForNewData(nextState);
        }

        return true;
    },

    getStateFromFlux() {
        var flux = this.getFlux(),
            id = flux.store('route').getParam('id'),
            data = flux.store('policy').getData(),
            resource = flux.store('resources').getResource('policy', id);

        var ret = {
            id: id,
            loading: data.loading,
            resource: resource.data,
            owner: resource.data.owner,
            annotations: resource.data.annotations,
            owned: data.owned,
            roles: data.roles,
            resources: data.resources
        };

        ret.loading.resource = resource.loading;
        ret.loading.owner = resource.loading;
        ret.loading.annotations = resource.loading;

        return ret;
    },

    render() {
        //TODO: policy loader
        //TODO in CLI: why not save text of policy as an annotation?
        var overviewTab = (
          <TabPane eventKey="overview" tab="Overview">
            <dl className="dl-horizontal">
              <dt>Owner</dt>
              <dd><RoleLink id={this.state.owner} /></dd>
            </dl>
          </TabPane>
        );

        var permissionsTab = this.permissionsTab(this.state.resource.id),
            membershipsTab = this.membershipsTab(this.state.resource.id),
            annotationsTab = this.annotationsTab(),
            ownedTab = this.ownedTab();

        var auditTab = (
          <TabPane eventKey="audit" tab="Recent Activity">
            <div className="audit auditGroup">
              <AuditTable roles={[this.state.resource.id]} tabview={true} />
            </div>
          </TabPane>
        );

        var tabs = compact([
            overviewTab,
            ownedTab,
            membershipsTab,
            permissionsTab,
            annotationsTab,
            auditTab
        ]);

        return (
          <div className="policy">
            <h2>Policy {this.state.resource.identifier}</h2>
            <TabbedArea defaultActiveKey="overview">
              {tabs}
            </TabbedArea>
          </div>
        );
    }
});
