var React = require('react'),
    Fluxxor = require('fluxxor'),
    Router = require('react-router');

import {groupBy, map} from 'lodash';

var FluxMixin = Fluxxor.FluxMixin(React),
    StoreWatchMixin = Fluxxor.StoreWatchMixin;

var SearchGroup = require('./group'),
    SearchGroupTitle = require('./group_title'),
    Refresh = require('../refresh/refresh');

export default React.createClass({
    displayName: 'Search',

    mixins: [FluxMixin, StoreWatchMixin('search', 'route'), Router.State],

    askForNewData(state) {
        this.getFlux().actions.search(state.query);
    },

    componentDidMount() {
        this.askForNewData(this.state);
    },

    componentWillUpdate(nextProps, nextState) {
        if (this.state.query !== nextState.query) {
            this.askForNewData(nextState);
        }

        return true;
    },

    getStateFromFlux() {
        var flux = this.getFlux(),
            data = flux.store('search').getData(),
            query = flux.store('route').getParam('query');

        return {
            loading: data.loading.results,
            results: data.results,
            query: query
        };
    },

    render() {
        var results = this.state.results;

        var grouped = groupBy(results, function(r) {
            return r.id.split(':')[1];
        });

        // Don't care about these
        delete grouped['environment-variables'];
        delete grouped.notification;
        delete grouped.queue;

        var groups = map(grouped, function(items, key) {
            return <SearchGroup kind={key} items={items} />;
        });

        var scores = {
            policy: 1,
            layer: 2,
            group: 3,
            host: 4,
            user: 5,
            variable: 6,
            /*eslint-disable camelcase */
            key_pair: 7
            /*eslint-enable camelcase */
        };

        groups.sort(function(a, b) {
            return (
                (scores[a.props.kind] || 100) -
                    (scores[b.props.kind] || 100)
            );
        });

        var toc = groups.map(function(g) {
            var gid = '#search-group-' + g.props.kind;

            return (
              <div className="toc-item">
                <a href={gid}>
                  <SearchGroupTitle kind={g.props.kind}
                                    items={g.props.items} />
                </a>
              </div>
            );
        });

        var heading;

        if (this.state.loading) {
            heading = 'Searching for "' + this.state.query + '"';
        } else {
            heading = 'Found ' + this.state.results.length +
                ' resources matching "' + this.state.query + '"';
        }

        return (
          <div id="searchResults">
            <div className="searchResults">
              <h3>{heading}<Refresh show={this.state.loading} /></h3>
              <div className="search-results-toc">
                {toc}
              </div>
              <div className="search-results">
                {groups}
              </div>
            </div>
          </div>
        );
    }
});
