var React = require('react');

var Chart = require('../chart/chart'),
    ChartHelperMixin = require('../chart/chart_helper_mixin'),
    Refresh = require('../refresh/refresh');

export default React.createClass({
    displayName: 'UserActivity',

    mixins: [ChartHelperMixin],

    getDefaultProps() {
        return {
            audit: [],
            options: {
                legend: {
                    logins: 'Logins',
                    sudo: 'Sudo calls',
                    reads: 'Secret reads',
                    updates: 'Secret updates',
                    warnings: 'Warnings'
                },
                axis: {
                    y: {
                        label: 'Value'
                    }
                }
            }
        };
    },

    getDefaultItem() {
        return {
            logins: 0,
            sudo: 0,
            reads: 0,
            updates: 0,
            warnings: 0
        };
    },

    getEventType(e) {
        if (e.hasOwnProperty('facility') && e.facility === 'ssh') {
            if (e.action === 'sudo' && e.allowed === true) {
                return 'sudo';

            } else if (e.action === 'login' && e.allowed === true) {
                return 'logins';
            }
        }

        if (e.hasOwnProperty('error') || e.allowed === false) {
            return 'warnings';
        }

        if (e.action==='check' &&
            e.hasOwnProperty('allowed') &&
            e.allowed === true) {

            if (e.hasOwnProperty('role') &&
                e.role === this.props.roleid) {

                if (e.hasOwnProperty('resource') &&
                    e.resource.split(':')[1] === 'variable') {

                    if (e.privilege === 'update') {
                        return 'updates';

                    } else if (e.privilege === 'execute') {
                        return 'reads';
                    }
                }
            }
        }

        return null;
    },

    getInitialState() {
        return {showActivity: false};
    },

    toggleActivity(e) {
        e.preventDefault();

        this.setState({showActivity: !this.state.showActivity});
    },

    render() {
        var data = this.getData(this.props.audit);
        var activityBody = [];

        var msg = this.state.showActivity ? 'Hide' : 'Show graph';

        if (this.state.showActivity) {
            activityBody.push(
                <div className="b-user-activity__graph">
                  <Chart options={this.props.options}
                         data={data} />
                </div>
            );
        }

        return (
          <div className="b-user-activity">
            <h2>Activity<Refresh show={this.props.isLoading} /></h2>
            <a href="#" onClick={this.toggleActivity}>{msg}</a>
            {activityBody}
          </div>
        );
    }
});
