var React = require('react'),
    Fluxxor = require('fluxxor');

import {compact} from 'lodash';

var FluxMixin = Fluxxor.FluxMixin(React),
    StoreWatchMixin = Fluxxor.StoreWatchMixin;

var TabbedArea = require('react-bootstrap/lib/TabbedArea'),
    TabPane = require('react-bootstrap/lib/TabPane');

var TabMixin = require('../generic/tab_mixin'),
    AuditTable = require('../audit/table'),
    Breadcrumbs = require('../generic/breadcrumbs'),
    Annotations = require('../generic/annotations'),
    FoldableAuditSection = require('../generic/foldable_audit_section'),
    Refresh = require('../refresh/refresh'),
    RoleGraph = require('../graph/graph');

import {getTabname} from '../../utils';

var PubkeysTabContent = require('./pubkeys'),
    Details = require('./details'),
    Activity = require('./activity');

export default React.createClass({
    displayName: 'UserView',

    mixins: [FluxMixin, StoreWatchMixin('audit', 'user', 'resources', 'route'), TabMixin],

    askForNewData(state) {
        var actions = this.getFlux().actions;

        actions.audit.loadForRole('user', state.id);
        actions.audit.loadForResource('user', state.id);
        actions.user.load(state.id);
        actions.resources.loadOne('user', state.id);
    },

    componentDidMount() {
        this.askForNewData(this.state);
    },

    componentWillUpdate(nextProps, nextState) {
        if (this.state.id !== nextState.id) {
            this.askForNewData(nextState);
        }

        return true;
    },

    getStateFromFlux() {
        var flux = this.getFlux(),
            id = flux.store('route').getParam('id'),
            audit = flux.store('audit').getEventFor('user', id),
            data = flux.store('user').getData(),
            resource = flux.store('resources').getResource('user', id);

        var ret = {
            id: id,
            loading: data.loading,
            user: data.user,
            owned: data.owned,
            roles: data.roles,
            owner: resource.data.owner,
            annotations: resource.data.annotations,
            resource: resource.data,
            pubkeys: data.pubkeys,
            audit: audit.data,
            resources: data.resources
        };

        ret.loading.audit = audit.loading;
        ret.loading.owner = resource.loading;
        ret.loading.annotations = resource.loading;

        return ret;
    },

    render() {
        var pubkeysTab = (
          <TabPane eventKey="pubkeys"
                   tab={getTabname('Public Keys', this.state.pubkeys)}>
            <PubkeysTabContent login={this.state.user.login}
                               pubkeys={this.state.pubkeys} />
          </TabPane>
        );

        var permissionsTab = this.permissionsTab(this.state.user.roleid),
            membershipsTab = this.membershipsTab(this.state.user.roleid),
            ownedTab = this.ownedTab();

        var tabs = compact([
            pubkeysTab,
            ownedTab,
            membershipsTab,
            permissionsTab
        ]);

        var elems = [
            {url: '/ui/users', text: 'Users'},
            {url: '', text: this.state.user.login},
            {url: '', text: '(owned by ' + this.state.user.ownerid + ')'}
        ];

        return (
          <div className="b-user">
            <Breadcrumbs elems={elems} />
            <hr />
            <Activity audit={this.state.audit}
                      roleid={this.state.user.roleid}
                      isLoading={this.state.loading.user || this.state.loading.audit} />
            <hr />
            {FoldableAuditSection.warnings(this.state.audit, this.state.loading.audit)}
            <hr />
            <div className="row">
              <div className="col-md-6">
                <Details owner={this.state.owner}
                         user={this.state.user}
                         isLoading={this.state.loading.user || this.state.loading.owner} />
              </div>
              <div className="col-md-6">
                <h2>Annotations<Refresh show={this.state.loading.annotations} /></h2>
                <Annotations annotations={this.state.annotations} />
              </div>
            </div>
            <hr />
            <h2>Other</h2>
            <TabbedArea>
              {tabs}
            </TabbedArea>
            <hr />
            <RoleGraph height="400"
                       kind="user"
                       id={this.state.id} />
            <hr />
            <AuditTable events={this.state.audit}
                        caption="Recent Activity"
                        isLoading={this.state.loading.audit} />
          </div>
        );
    }
});
