var React = require('react'),
    Fluxxor = require('fluxxor');

var FluxMixin = Fluxxor.FluxMixin(React),
    StoreWatchMixin = Fluxxor.StoreWatchMixin;

var AuditTable = require('../audit/table'),
    Annotations = require('../generic/annotations'),
    Breadcrumbs = require('../generic/breadcrumbs'),
    FoldableAuditSection = require('../generic/foldable_audit_section'),
    Refresh = require('../refresh/refresh');

var Activity = require('./activity'),
    Updaters = require('./updaters'),
    Fetchers = require('./fetchers'),
    Details = require('./details');

export default React.createClass({
    displayName: 'VariableView',

    mixins: [FluxMixin, StoreWatchMixin('audit', 'variable', 'resources', 'route')],

    askForNewData(state) {
        var actions = this.getFlux().actions;

        actions.audit.loadForResource('variable', state.id);
        actions.variable.load(state.id);
        actions.resources.loadOne('variable', state.id);
    },

    componentDidMount() {
        this.askForNewData(this.state);
    },

    componentWillUpdate(nextProps, nextState) {
        if (this.state.id !== nextState.id) {
            this.askForNewData(nextState);
        }

        return true;
    },

    getStateFromFlux() {
        var flux = this.getFlux(),
            id = flux.store('route').getParam('id'),
            audit = flux.store('audit').getEventFor('variable', id),
            data = flux.store('variable').getData(),
            resource = flux.store('resources').getResource('variable', id);

        var ret = {
            id: id,
            loading: data.loading,
            variable: data.variable,
            owner: resource.data.owner,
            annotations: resource.data.annotations,
            audit: audit.data,
            fetchers: data.fetchers,
            updaters: data.updaters
        };

        ret.loading.audit = audit.loading;
        ret.loading.owner = resource.loading;
        ret.loading.annotations = resource.loading;

        return ret;
    },

    render() {
        var elems = [
            {url: '/ui/variables', text: 'Variables'},
            {url: '', text: this.state.variable.id},
            {url: '', text: '(owned by ' + this.state.variable.ownerid + ')'}
        ];

        return (
          <div className="b-variable">
            <Breadcrumbs elems={elems} />
            <hr />
            <Activity audit={this.state.audit}
                      isLoading={this.state.loading.audit} />
            <hr />
            <div className="row">
              <div className="col-md-6">
                <Updaters data={this.state.updaters}
                          isLoading={this.state.loading.updaters} />
              </div>
              <div className="col-md-6">
                <Fetchers data={this.state.fetchers}
                          isLoading={this.state.loading.fetchers} />
              </div>
            </div>
            <hr />
            {FoldableAuditSection.updates(this.state.audit, this.state.loading.audit)}
            <hr />
            {FoldableAuditSection.warnings(this.state.audit, this.state.loading.audit)}
            <hr />
            <div className="row">
              <div className="col-md-6">
                <Details owner={this.state.owner}
                         variable={this.state.variable}
                         isLoading={this.state.loading.variable || this.state.loading.owner} />
              </div>
              <div className="col-md-6">
                <h2>Annotations<Refresh show={this.state.loading.annotations} /></h2>
                <Annotations annotations={this.state.annotations} />
              </div>
            </div>
            <hr />
            <AuditTable events={this.state.audit}
                        caption="Recent Activity"
                        isLoading={this.state.loading.audit} />
          </div>
        );
    }
});
