import React from 'react';
import {Route, DefaultRoute, NotFoundRoute} from 'react-router';

import App from './components/app/app';
import Wrapper from './components/app/wrapper';
import {NotFound} from './components/notfound/notfound';
import {Login} from './components/auth/login';
import {Logout} from './components/auth/logout';
import Dashboard from './components/dashboard/dashboard';
import Search from './components/search/search';

import UserList from './components/user/list';
import UserView from './components/user/view';
import GroupList from './components/group/list';
import GroupView from './components/group/view';
import HostList from './components/host/list';
import HostView from './components/host/view';
import LayerList from './components/layer/list';
import LayerView from './components/layer/view';
import VariableList from './components/variable/list';
import VariableView from './components/variable/view';
import PolicyList from './components/policy/list';
import PolicyView from './components/policy/view';
import CustomList from './components/custom/list';
import CustomView from './components/custom/view';

export default ([
  <Route handler={Wrapper}>
    <Route path="/ui/?" handler={App}>
      <Route name="dashboard" path="/ui/?" handler={Dashboard} />

      <Route name="users" path="/ui/users/?" handler={UserList} />
      <Route name="user" path="/ui/users/:id/?" handler={UserView} />

      <Route name="groups" path="/ui/groups/?" handler={GroupList} />
      <Route name="group" path="/ui/groups/:id/?" handler={GroupView} />

      <Route name="hosts" path="/ui/hosts/?" handler={HostList} />
      <Route name="host" path="/ui/hosts/:id/?" handler={HostView} />

      <Route name="layers" path="/ui/layers/?" handler={LayerList} />
      <Route name="layer" path="/ui/layers/:id/?" handler={LayerView} />

      <Route name="variables" path="/ui/variables/?" handler={VariableList} />
      <Route name="variable" path="/ui/variables/:id/?" handler={VariableView} />

      <Route name="policies" path="/ui/policies/?" handler={PolicyList} />
      <Route name="policy" path="/ui/policies/:id/?" handler={PolicyView} />

      <Route name="custom-types" path="/ui/custom-types/?" handler={CustomList} />
      <Route name="custom-type" path="/ui/custom-type/:kind/:id/?" handler={CustomView} />

      <Route name="search" path="/ui/search/:query/?" handler={Search} />

      <DefaultRoute handler={Dashboard} />
    </Route>

    <Route name="login" path="/ui/login/?" handler={Login} />

    <Route name="logout" path="/ui/logout/?" handler={Logout} />

    <NotFoundRoute handler={NotFound} />
  </Route>
]);
