import {createStore} from 'fluxxor';
import c from '../constants';

export default createStore({
    initialize() {
        this.error = false;
        this.loading = false;
        this.graphs = {};

        this.bindActions(
            c.GRAPH.LOAD, this.onLoad,
            c.GRAPH.LOAD_SUCCESS, this.onLoadSuccess,
            c.GRAPH.LOAD_FAIL, this.onLoadFail
        );
    },

    onLoad() {
        this.error = false;
        this.loading = true;
        this.emit('change');
    },

    onLoadSuccess(payload) {
        this.error = false;
        this.loading = false;

        if (typeof this.graphs[payload.kind] !== 'object') {
            this.graphs[payload.kind] = {};
        }

        if (typeof this.graphs[payload.kind][payload.id] !== 'object') {
            this.graphs[payload.kind][payload.id] = payload.res.body.graph;
        }

        this.emit('change');
    },

    onLoadFail() {
        this.error = true;
        this.loading = false;
        this.emit('change');
    },

    getGraph(kind, id) {
        return {
            loading: this.loading,
            error: this.error,
            data: (this.graphs[kind] || {})[id] || []
        };
    }
});
