import {createStore} from 'fluxxor';
import {c as generic} from '../clients/generic';
import {c as members} from '../clients/members';
import {HOST as constants} from '../constants';

var fieldMapping = {
    ATTRIBUTES: 'host',
    OWNED_RESOURCES: 'owned',
    ALL_ROLES: 'roles',
    RESOURCES: 'resources',
    MEMBERS_EXECUTE: 'executors',
    MEMBERS_UPDATE: 'updaters'
};

export default createStore({
    initialize() {
        this.data = this._fresh();

        this.bindActions(
            constants.LOAD, this.onLoad,
            constants.LOAD_SUCCESS, this.onLoadSuccess,
            constants.LOAD_FAIL, this.onLoadFail
        );
    },

    _fresh() {
        return {
            host: {},
            loading: {
                host: true,
                owned: true,
                roles: true,
                resources: true
            }
        };
    },

    onLoad(payload) {
        if (typeof payload.type === 'undefined') {
            this.data = this._fresh();
            this.data.identifier = payload.id;
            this.emit('change');
        } else if (payload.id === this.data.identifier) {
            this.turn(payload.type, true);
            this.emit('change');
        }
    },

    onLoadSuccess(payload) {
        if (payload.id === this.data.identifier) {
            this.turn(payload.type, false);

            this.setData(payload);
            this.emit('change');
        }
    },

    onLoadFail(payload) {
        this.turn(payload.type, false);
    },

    turn(type, state) {
        var fields = fieldMapping[type];

        if (typeof fields === 'object') {
            fields.forEach((field) => {
                this.data.loading[field] = state;
            });
        } else {
            this.data.loading[fields] = state;
        }
    },

    notInternalRole(item) {
        return item.split(':')[1] !== '@';
    },

    setData(payload) {
        switch (payload.type) {
        case generic.ATTRIBUTES:
            this.data.host = payload.res.body;
            break;

        case generic.OWNED_RESOURCES:
            this.data.owned = payload.res.body;
            break;

        case generic.ALL_ROLES:
            this.data.roles = payload.res.body;
            break;

        case members.MEMBERS_EXECUTE:
            this.data.executors = payload.res.body.filter(this.notInternalRole);
            break;

        case members.MEMBERS_UPDATE:
            this.data.updaters = payload.res.body.filter(this.notInternalRole);
            break;

        case 'RESOURCES':
            this.data.resources = payload.res.body;
            break;

        default:
        }
    },

    getData() {
        return this.data;
    }
});
