import {createStore} from 'fluxxor';
import {POLICY as constants} from '../constants';

var fieldMapping = {
    OWNED_RESOURCES: 'owned',
    ALL_ROLES: 'roles',
    RESOURCES: 'resources'
};

export default createStore({
    initialize() {
        this.data = this._fresh();

        this.bindActions(
            constants.LOAD, this.onLoad,
            constants.LOAD_SUCCESS, this.onLoadSuccess,
            constants.LOAD_FAIL, this.onLoadFail
        );
    },

    _fresh() {
        return {
            loading: {
                owned: true,
                role: true,
                resources: true
            }
        };
    },

    onLoad(payload) {
        if (typeof payload.type === 'undefined') {
            this.data = this._fresh();
            this.data.identifier = payload.id;
            this.emit('change');
        } else if (payload.id === this.data.identifier) {
            this.turn(payload.type, true);
            this.emit('change');
        }
    },

    onLoadSuccess(payload) {
        if (payload.id === this.data.identifier) {
            this.turn(payload.type, false);

            this.setData(payload);
            this.emit('change');
        }
    },

    onLoadFail(payload) {
        this.turn(payload.type, false);
    },

    turn(type, state) {
        var fields = fieldMapping[type];

        if (typeof fields === 'object') {
            fields.forEach((field) => {
                this.data.loading[field] = state;
            });
        } else {
            this.data.loading[fields] = state;
        }
    },

    setData(payload) {
        switch (payload.type) {
        case 'OWNED_RESOURCES':
            this.data.owned = payload.res.body;
            break;

        case 'ALL_ROLES':
            this.data.roles = payload.res.body;
            break;

        case 'RESOURCES':
            this.data.resources = payload.res.body;
            break;

        default:
        }
    },

    getData() {
        return this.data;
    }
});
