import {createStore} from 'fluxxor';
import _ from 'lodash';
import c from '../constants';

export default createStore({
    initialize() {
        this.error = false;
        this.loading = false;
        this.resources = {};

        this.bindActions(
            c.RESOURCE.LOAD, this.onLoad,
            c.RESOURCE.LOAD_SUCCESS, this.onLoadSuccessOne,
            c.RESOURCE.LOAD_FAIL, this.onLoadFail,
            c.RESOURCES.LOAD, this.onLoad,
            c.RESOURCES.LOAD_SUCCESS, this.onLoadSuccess,
            c.RESOURCES.LOAD_FAIL, this.onLoadFail
        );
    },

    onLoad() {
        this.error = false;
        this.loading = true;
        this.emit('change');
    },

    _parse(resource) {
        var tokens = resource.id.split(':');

        return {
            id: resource.id,
            kind: tokens[1],
            identifier: tokens[tokens.length-1],
            owner: resource.owner,
            annotations: resource.annotations || [],
            permission: resource.permission || []
        };
    },

    onLoadSuccessOne(payload) {
        var res = this._parse(payload.res.body),
            i;

        this.error = false;
        this.loading = false;

        if (!_.isArray(this.resources[res.kind])) {
            this.resources[res.kind] = [];
        }

        i = _.findIndex(this.resources[res.kind], 'id', res.id);

        if (i > -1) {
            this.resources[res.kind][i] = res;
        } else {
            this.resources[res.kind].push(res);
        }

        this.emit('change');
    },

    onLoadSuccess(payload) {
        var res = _.map(payload.res.body, (resource) => {
            return this._parse(resource);
        });

        this.error = false;
        this.loading = false;

        res = _.groupBy(res, 'kind');

        _.forIn(res, (value, key) => {
            this.resources[key] = value;
        });

        this.emit('change');
    },

    onLoadFail() {
        this.error = true;
        this.loading = false;
    },

    getResources(kind) {
        if (kind === 'custom-type') {
            return this.getCustomResources();
        } else {
            return {
                loading: this.loading,
                error: this.error,
                data: this.resources[kind] || []
            };
        }
    },

    getCustomResources() {
        var knownTypes = ['user', 'group', 'host', 'layer', 'variable', 'policy'],
            customTypes = _.difference(_.keys(this.resources), knownTypes),
            resources = _.flatten(_.map(customTypes, _.propertyOf(this.resources)));

        return {
            loading: this.loading,
            error: this.error,
            data: resources || []
        };
    },

    getResource(kind, id) {
        return {
            loading: this.loading,
            error: this.error,
            data: _.find(this.resources[kind], 'identifier', id) || {}
        };
    }
});
