import {createStore} from 'fluxxor';
import {c as generic} from '../clients/generic';
import {c as members} from '../clients/members';
import {VARIABLE as constants} from '../constants';

var fieldMapping = {
    ATTRIBUTES: 'variable',
    MEMBERS_EXECUTE: 'fetchers',
    MEMBERS_UPDATE: 'updaters'
};

export default createStore({
    initialize() {
        this.data = this._fresh();

        this.bindActions(
            constants.LOAD, this.onLoad,
            constants.LOAD_SUCCESS, this.onLoadSuccess,
            constants.LOAD_FAIL, this.onLoadFail
        );
    },

    _fresh() {
        return {
            variable: {
                userid: ''
            },
            loading: {
                variable: true,
                fetchers: true,
                updaters: true
            }
        };
    },

    onLoad(payload) {
        if (typeof payload.type === 'undefined') {
            this.data = this._fresh();
            this.data.identifier = payload.id;
            this.emit('change');
        } else if (payload.id === this.data.identifier) {
            this.turn(payload.type, true);
            this.emit('change');
        }
    },

    onLoadSuccess(payload) {
        if (payload.id === this.data.identifier) {
            this.turn(payload.type, false);

            this.setData(payload);
            this.emit('change');
        }
    },

    onLoadFail(payload) {
        this.turn(payload.type, false);
    },

    turn(type, state) {
        var fields = fieldMapping[type];

        if (typeof fields === 'object') {
            fields.forEach((field) => {
                this.data.loading[field] = state;
            });
        } else {
            this.data.loading[fields] = state;
        }
    },

    setData(payload) {
        switch (payload.type) {
        case generic.ATTRIBUTES:
            this.data.variable = payload.res.body;
            break;

        case members.MEMBERS_EXECUTE:
            this.data.fetchers = payload.res.body;
            break;

        case members.MEMBERS_UPDATE:
            this.data.updaters = payload.res.body;
            break;

        default:
        }
    },

    getData() {
        return this.data;
    }
});
