#
# Copyright (C) 2013 Conjur Inc
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of
# this software and associated documentation files (the "Software"), to deal in
# the Software without restriction, including without limitation the rights to
# use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
# the Software, and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
# FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
# COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
# IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
class Conjur::Command::UI < Conjur::Command
  desc "Launch a UI"
  arg_name "root-path"
  Conjur::CLI.command :ui do |c|
    c.desc "Port to run the webserver on."
    c.flag ["p", "port"]

    c.desc "Run on headless server (don't launch the browser)."
    c.switch [:headless]
      
    c.action do |global_options,options,args|
      root = args.pop || File.expand_path('../../../app/build', File.dirname(__FILE__))
      port = options[:port]

      #TODO: login page
      api # just initialize API so if user is not logged in it will be asked for credentials *before* server is launched
      @@api=nil # reset to avoid conflicts on server launch

      # ugly hack to override Configuration.env issues which are not working any more
      # after removal of sticky configurations support
      ENV["RACK_ENV"]="production"

      require 'conjur/webserver/server'
      server = Conjur::WebServer::Server.new
      server.set_port port if port
      thread = Thread.new do
        server.start(root)
      end
      sleep 0.5
      server.open options[:headless]
      thread.join
    end
  end
end
